<?php
/**
*
*/
class datamodel_Core {

	/**
	 * Matches $hostname with data in $page_graph.
	 * @param array         $page_graph Page graph data.
	 * @param string        $hostname   Host name.
	 * @param callable|null $match_func Function to be used when matching data.
	 * @return string|boolean Host name if a match is found. Otherwise false.
	 */
	public static function filter_host_by_page(
		array $page_graph, $hostname, $match_func = null
	) {
		// Default to match_string function.
		if ($match_func === null)
			$match_func = array('datamodel', 'match_string');

		foreach ($page_graph as $g) {
			if (!isset($g['host_name']))
				continue;

			// $g['host_name'] contains list of hosts or regexp.
			// Replace spaces with underscore.
			$graph_host_name = str_replace(' ', '_', $g['host_name']);

			if (call_user_func($match_func, $hostname, $graph_host_name))
				return $hostname;
		}
		return false;
	}

	/**
	 * Matches $hostname $servicename with the names given in $graph.
	 * @param array  $graph       Graph data from a single page graph entry.
	 * @param string $hostname    Host name.
	 * @param string $servicename Service name.
	 * @param callable|null $match_func Function to be used when matching data.
	 * @return array|boolean Array with graph data (host name, service name and
	 *                       source which always equals null in this case) or
	 *                       false if no match is found.
	 */
	public static function filter_service_by_graph(
		$graph, $hostname, $servicename, $match_func = null
	) {
		if (!isset($graph['host_name']) || !isset($graph['service_desc']))
			return false;

		// Default to match_string function.
		if ($match_func === null)
			$match_func = array('datamodel', 'match_string');

		$data = array();
		$graph_host_name = str_replace(' ', '_', $graph['host_name']);
		$graph_srv_desc = str_replace(' ', '_', $graph['service_desc']);
		if (call_user_func($match_func, $hostname, $graph_host_name) &&
		    call_user_func($match_func, $servicename, $graph_srv_desc))
		{
			$data['service_desc'] = $graph_srv_desc;
			$data['host_name']    = $graph_host_name;
			$data['source']       = null;
			return $data;
		}
		return false;
	}

	/**
	 * Read XML file $xmlfile.
	 * @param string $xmlfile
	 * @param array& $ds       Data source reference.
	 * @param array& $macro    Macro reference.
	 *
	 * @throws Kohana_Exception on XML data errors.
	 * @return SimpleXMLElement XML data.
	 */
	public static function read_xml($xmlfile, &$ds, &$macro) {
		if (!file_exists($xmlfile))
			throw new Kohana_Exception('error.xml-not-found', $xmlfile);

		libxml_use_internal_errors(true);
		libxml_clear_errors(true);
		if (!$xml = simplexml_load_file($xmlfile)) {
			$errors = '<br>';
			foreach(libxml_get_errors() as $error) {
				$errors .= $error->message . "<br>";
			}
			throw new Kohana_Exception(
				'error.xml-generic_error', $xmlfile, $errors
			);
		}

		$macro = array();
		$macro['AUTH_SERVICEDESC'] = '';
		$macro['AUTH_HOSTNAME']    = '';
		if (!isset($xml->XML->VERSION)) {
			throw new Kohana_Exception(
				'error.xml-structure-without-version-tag', $xmlfile
			);
		}

		foreach ($xml as $key => $val) {
			if (preg_match('/^NAGIOS_(.*)$/', $key, $match)) {
				$key = $match[1];
				$macro[$key] = (string) $val;
			}
		}

		$i = 0;
		$ds = array();
		foreach ($xml->DATASOURCE as $datasource) {
			foreach ($datasource  as $key => $val) {
				$ds[$i][$key] = (string) $val;
			}
			$i++;
		}
		return $xml;
	}

	/**
	 * Match with preg_match.
	 * @return boolean True if expression $regex matches content in $haystack.
	 *                 Otherwise false.
	 */
	public static function match_regex($haystack, $regex) {
		return preg_match('/' . $regex . '/', $haystack) == 1;
	}

	/**
	 * Match string.
	 * @return boolean True if $haystack is found within in $needle.
	 *                 Otherwise false.
	 */
	public static function match_string($needle, $haystack) {
		return strpos($haystack, $needle) !== false;
	}
}
